/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect } from "react";
import moment from 'moment';
// Maerial ui components
import Card from "@material-ui/core/Card";
import CardContent from "@material-ui/core/CardContent";
import Typography from "@material-ui/core/Typography";
import Button from "@material-ui/core/Button";
import Paper from "@material-ui/core/Paper";
import Grid from "@material-ui/core/Grid";
import Divider from "@material-ui/core/Divider";
import ListRoundedIcon from "@material-ui/icons/ListRounded";
import GetAppRoundedIcon from "@material-ui/icons/GetAppRounded";
import LaunchRoundedIcon from "@material-ui/icons/LaunchRounded";
import RemoveRoundedIcon from "@material-ui/icons/RemoveRounded";
import RestoreRoundedIcon from "@material-ui/icons/RestoreRounded";
import { useStyles } from "../../Styles/DataCard";
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogContentText from '@material-ui/core/DialogContentText';
import DialogTitle from '@material-ui/core/DialogTitle';
import CircularProgress from '@material-ui/core/CircularProgress';
//imports from local files
import { GetAData, MethodsToComplain } from "../../../Filters";
import { Link } from "react-router-dom";
import { CardActions } from "@material-ui/core";
import { apiDomain } from '../../../../env';
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import SnackBar from '../../../../Components/SnackbarComponent';
import { getRequest, postRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
//styles

const PORTAL_MEC = "https://plataformaintegrada.mec.gov.br/";

const CollectionCard = ({ match }) => {
    const classes = useStyles();


    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [item, setItem] = useState({}); //Necessary to consult the API, data
    const [complainedObject, setComplainedObject] = useState(null);//stores the data from complained object
    const [contentModal, setContentModal] = useState("");
    const [titleModal, setTitleModal] = useState("");

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    const [loadingObject, setLoadingObject] = useState(false);
    const [open, setOpen] = React.useState(false);
    const [scroll, setScroll] = React.useState('paper');

    const handleClose = () => {
        setOpen(false);
    };

    const descriptionElementRef = React.useRef(null);
    React.useEffect(() => {
        if (open) {
            const { current: descriptionElement } = descriptionElementRef;
            if (descriptionElement !== null) {
                descriptionElement.focus();
            }
        }
    }, [open]);

    const convertToLink = (type, id) => {
        switch (type) {
            case "LearningObject":
                return `recurso?id=${id}/`;
            case "User":
                return `usuario-publico/${id}/`;
            default:
                return "";
        }
    };

    const handleClickOpen = (scrollType) => async () => {
        setLoadingObject(true);
        setOpen(true);
        setScroll(scrollType);

        if (!complainedObject) {
            getRequest(
                GetAData("learning_objects", item.complainable_id),
                (data, header) => {
                    HandleContentModal(data);
                    setComplainedObject(data)
                    setLoadingObject(false);            
                },
                (error) => {
                    HandleContentModal();
                    setLoadingObject(false);
                }
            )
        } else {
            HandleContentModal(complainedObject);
            setLoadingObject(false);
        }
    };

    const ComplaintStatus = (status) => {
        switch (status) {
            case "accepted":
                return (
                    <Paper
                        className={classes.marginTop}
                        style={{
                            textAlign: "center",
                            padding: "0.5em",
                            backgroundColor: "#FA8072",
                            fontWeight: "500",
                            color: "#FFFAFA",
                        }}
                    >
                        REMOVIDO
                    </Paper>
                );
            case "complained":
                return (
                    <Paper
                        className={classes.marginTop}
                        style={{
                            textAlign: "center",
                            padding: "0.5em",
                            backgroundColor: "#FF8C00",
                            fontWeight: "500",
                            color: "#FFFAFA",
                        }}
                    >
                        PENDENTE
                    </Paper>
                );
            case "rejected":
                return (
                    <Paper
                        className={classes.marginTop}
                        style={{
                            textAlign: "center",
                            padding: "0.5em",
                            backgroundColor: "#228B22",
                            fontWeight: "500",
                            color: "#FFFAFA",
                        }}
                    >
                        AVALIADO
                    </Paper>
                );
            default:
                return "NOTHING";
        }
    };

    const Actions = (status) => {
        switch (status) {
            case "accepted":
                return (
                    "Este recurso já foi avaliado"
                );
            case "complained":
                return (
                    <>
                        <Grid
                            container
                            className={classes.marginTop}
                            direction="row"
                            justify="space-between"
                            alignItems="center"
                        >
                            <Grid item>
                                <Typography
                                    color="initial"
                                    className={classes.subTitle}
                                    variant="h6"
                                >
                                    Remover o recurso e notificar o autor
                                </Typography>
                            </Grid>
                            <Grid item>
                                <Button
                                    style={{
                                        color: "#FFFAFA",
                                        backgroundColor: "#FA8072",
                                        fontWeight: "500",
                                    }}
                                    variant="contained"
                                    onClick={() => HandleComplainSubmmit("accept")}
                                    startIcon={
                                        <RemoveRoundedIcon style={{ fill: "#FFFAFA" }} />
                                    }
                                >
                                    Remover
                                </Button>
                            </Grid>
                        </Grid>

                        <Divider className={classes.marginTop} />

                        <Grid
                            container
                            className={classes.marginTop}
                            direction="row"
                            justify="space-between"
                            alignItems="center"
                        >
                            <Grid item>
                                <Typography
                                    color="initial"
                                    className={classes.subTitle}
                                    variant="h6"
                                >
                                    Reativar o recurso na plataforma
                                        </Typography>
                            </Grid>
                            <Grid item>
                                <Button
                                    style={{
                                        color: "#FFFAFA",
                                        backgroundColor: "#228B22",
                                        fontWeight: "500",
                                    }}
                                    variant="contained"
                                    onClick={() => HandleComplainSubmmit("reject")}
                                    startIcon={
                                        <RestoreRoundedIcon style={{ fill: "#FFFAFA" }} />
                                    }
                                >
                                    Ativar
                                </Button>
                            </Grid>
                        </Grid>
                    </>
                );
            case "rejected":
                return (
                    "Este recurso já foi avaliado"
                );
            default:
                return "NOTHING";
        }
    }

    const HandleContentModal = (data) => {
        const ok = data ? true : false;

        if (ok) {
            const importantData = [
                {
                    subTitle: "ID",
                    prop: data.id,
                },
                {
                    subTitle: "Nome",
                    prop: data.name,
                },
                {
                    subTitle: "Descrição",
                    prop: data.description,
                },
                {
                    subTitle: "Dono(a)",
                    prop: data.publisher.name,
                },
                {
                    subTitle: "Autor(a)",
                    prop: data.author,
                },
                {
                    subTitle: "Score",
                    prop: data.score,
                },
                {
                    subTitle: "Status",
                    prop: data.state,
                },
                {
                    subTitle: "Criado em",
                    prop: data.created_ats
                },
                {
                    subTitle: "Atualizado em",
                    prop: DisplayDate(data.updated_at)
                },
                {
                    subTitle: "Criado em",
                    prop: DisplayDate(data.created_at)
                },
                {
                    subTitle: "Visualizações",
                    prop: data.views_count,
                },
                {
                    subTitle: "Likes",
                    prop: data.likes_count,
                },
                {
                    subTitle: "Avaliação média",
                    prop: data.review_average,
                },
                {
                    subTitle: "Likes",
                    prop: data.likes_count,
                },
                {
                    subTitle: "Linguagem",
                    prop: (
                        <div
                            style={{
                                display: "flex",
                                flexDirection: "row",
                                flexWrap: "wrap",
                                justifyContent: "Space-between",
                            }}
                        >
                            {data.language.map((lan, index) => (
                                <Paper
                                    elevation={3}
                                    key={index}
                                    style={{
                                        borderRadius: 18,
                                        padding: "0.8em",
                                        marginBottom: "1em",
                                        marginRight: "1em",
                                        backgroundColor: "#D3D3D3",
                                        fontSize: 14,
                                    }}
                                >
                                    {lan.name}
                                </Paper>
                            ))}
                        </div>
                    ),
                },
                {
                    subTitle: "Tipo de recurso",
                    prop: data.object_type,
                },
            ]

            setContentModal(
                <Grid container>
                    <Grid item justify="flex-start" alignItems="center" alignContent="center">
                        {data.thumbnail ? (
                            <Grid item style={{ marginTop: "1em", marginBottom: "1em", alignSelf: "center" }}>
                                <a target="_blank" href={apiDomain + data.thumbnail}>
                                    {" "}
                                    <img src={apiDomain + data.thumbnail} alt="" />{" "}
                                </a>
                            </Grid>
                        ) : null}
                        {importantData.map((info, index) => (
                            <Grid item className={classes.displayColumn} key={index}>
                                <Typography color="textPrimary">
                                    {info.subTitle}
                                </Typography>
                                <Typography color="textSecondary">
                                    {info.prop === null ? "Sem dados" : info.prop}
                                </Typography>
                            </Grid>
                        ))}
                    </Grid>
                </Grid>
            )
            setTitleModal(data.name)

        } else {
            setTitleModal("Indisponível");
            setContentModal("Não foi possível encontrar!")
        }
    }

    const DisplayDate = (date) => {
        const convertedData = moment.utc(date);
        return moment(convertedData).format("LLL").toString();
    }

    const HandleComplainSubmmit = async (method) => {
        postRequest(
            MethodsToComplain("complaints", match.params.id, method),
            {},
            (data) => {
                HandleSnack('Alteração feito com sucesso', true, 'success', '#228B22')
            },
            (error) => {
                HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
            }
        )
    }

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    useEffect(() => {
        getRequest(
            GetAData("complaints", match.params.id),
            (data, header) => {
                setItem(data)
                setIsLoaded(true);
                setError(false);
            },
            (error) => {
                setError(true);
            }
        )   
    }, []);

    if (error) {
        return <div>Houve um erro</div>;
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..."/>
    } else {
        const DATA = [
            {
                subTitle: "ID",
                prop: item.id,
            },
            {
                subTitle: "State",
                prop: item.state,
            },
            {
                subTitle: "Descrição",
                prop: item.description,
            },
            {
                subTitle: "Status",
                prop: item.complaint_reason.status,
            },
            {
                subTitle: "Motivo",
                prop: item.complaint_reason.reason,
            },
            {
                subTitle: "Criado",
                prop: DisplayDate(item.created_at),
            },
            {
                subTitle: "User",
                prop: item.user_id,
            },
        ];

        return (
            <Grid
                container
                spacing={3}
                sm={12}
                xs={12}
            >
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() => setSnackInfo({
                        message: '',
                        icon: '',
                        open: false,
                        color: ''
                    })}
                />
                <Grid
                    item
                    sm={8}
                    xs={12}
                >
                    <Card>
                        <CardContent>
                            <Grid container justify="space-between">
                                <Grid item>
                                    <Typography
                                        className={classes.title}
                                        color="inherit"
                                        gutterBottom
                                    >
                                        Denuncia:
                                    </Typography>
                                    <Typography
                                        className={classes.title}
                                        color="inherit"
                                        gutterBottom
                                    >
                                        Recurso #{item.complainable_id}
                                    </Typography>
                                </Grid>
                                <Grid item>
                                    <Link
                                        style={{ textDecoration: "none" }}
                                        to={`/admin/complaints`}
                                    >
                                        <Button
                                            startIcon={<ListRoundedIcon />}
                                            color="primary"
                                            variant="outlined"
                                        >
                                            Listar
                                        </Button>
                                    </Link>
                                </Grid>
                            </Grid>

                            {DATA.map((info, index) => (
                                <div className={classes.displayColumn} key={index}>
                                    <Typography color="initial" className={classes.subTitle}>
                                        {info.subTitle}
                                    </Typography>
                                    <Typography color="textSecondary">
                                        {info.prop === null ? "Sem dados" : info.prop}
                                    </Typography>
                                </div>
                            ))}
                        </CardContent>
                        <CardActions>
                            <Button
                                variant="contained"
                                color="primary"
                                startIcon={<GetAppRoundedIcon />}
                                onClick={handleClickOpen('paper')}
                            >
                                Veja o recurso
                            </Button>

                            <Button
                                style={{
                                    fontWeight: "500",
                                    backgroundColor: "#FA8072"
                                }}
                                variant="contained"
                                startIcon={<LaunchRoundedIcon style={{ fill: "#FFFAFA" }} />}
                            >
                                <a
                                    style={{ textDecoration: "none", color: "#FFFAFA" }}
                                    target="_blank"
                                    href={
                                        PORTAL_MEC +
                                        convertToLink(
                                            item.complainable_type,
                                            item.complainable_id
                                        )
                                    }
                                >
                                    MEC RED
                                </a>
                            </Button>
                        </CardActions>
                    </Card>
                </Grid>

                <Grid
                    item
                    sm={4}
                    xs={12}
                >
                    <Grid item>
                        <Card>
                            <CardContent>
                                <Typography variant="h5" component="h2">
                                    Denúncia #{item.id}
                                </Typography>

                                {ComplaintStatus(item.state)}

                                <Grid container className={classes.marginTop}>
                                    <Grid item>
                                        <Typography
                                            color="textSecondary"
                                            className={classes.subTitle}
                                            variant="h6"
                                        >
                                            Descrição
                                        </Typography>
                                        <Typography color="initial">{item.description}</Typography>
                                    </Grid>
                                </Grid>

                                <Grid
                                    container
                                    direction="row"
                                    justify="space-between"
                                    alignItems="center"
                                    className={classes.marginTop}
                                >
                                    <Grid item>
                                        <Typography
                                            color="textSecondary"
                                            className={classes.subTitle}
                                        >
                                            Data
                                        </Typography>
                                        <Typography color="initial">{item.created_at}</Typography>
                                    </Grid>

                                    <Grid item>
                                        <Typography
                                            color="textSecondary"
                                            className={classes.subTitle}
                                        >
                                            Denunciante
                                        </Typography>
                                        <Typography color="initial">{item.user_id}</Typography>
                                    </Grid>
                                </Grid>

                                <Divider className={classes.marginTop} />

                                <Grid container className={classes.marginTop}>
                                    <Grid item>
                                        <Typography
                                            color="textSecondary"
                                            className={classes.subTitle}
                                            variant="h6"
                                        >
                                            Motivo
                                        </Typography>
                                        <Typography color="initial">
                                            {item.complaint_reason.reason}
                                        </Typography>
                                    </Grid>
                                </Grid>

                                <Grid container className={classes.marginTop}>
                                    <Grid item>
                                        <Typography
                                            color="textSecondary"
                                            className={classes.subTitle}
                                            variant="h6"
                                        >
                                            Status
                                        </Typography>
                                        <Typography color="initial">
                                            {item.complaint_reason.status}
                                        </Typography>
                                    </Grid>
                                </Grid>
                            </CardContent>
                        </Card>
                    </Grid>

                    <Grid item className={classes.marginTop}>
                        <Card>
                            <CardContent>
                                <Typography variant="h5" component="h2">
                                    Ações
                                </Typography>
                                {
                                    Actions(item.state)
                                }
                            </CardContent>
                        </Card>
                    </Grid>
                </Grid>

                <Dialog
                    open={open}
                    onClose={handleClose}
                    scroll={scroll}
                    aria-labelledby="scroll-dialog-title"
                    aria-describedby="scroll-dialog-description"
                >
                    <DialogTitle id="scroll-dialog-title">
                        {
                            loadingObject ? <CircularProgress size={24} /> : titleModal
                        }
                    </DialogTitle>
                    <DialogContent dividers={scroll === 'paper'}>
                        <DialogContentText
                            id="scroll-dialog-description"
                            ref={descriptionElementRef}
                            tabIndex={-1}
                        >
                            {
                                loadingObject ? <CircularProgress size={24} /> : contentModal
                            }
                        </DialogContentText>
                    </DialogContent>
                    <DialogActions>
                        <Button onClick={handleClose} color="primary">
                            Fechar
                        </Button>
                    </DialogActions>
                </Dialog>
            </Grid>
        );
    }
};

export default CollectionCard;
