/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useContext } from 'react';
//imports material ui componets
import Card from "@material-ui/core/Card";
import CardContent from "@material-ui/core/CardContent";
import CardAction from '@material-ui/core/CardActions';
import { Typography, TextField, Button, Grid } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import AddRoundedIcon from '@material-ui/icons/AddRounded';
import MenuItem from "@material-ui/core/MenuItem";
import ListRoundedIcon from '@material-ui/icons/ListRounded';
//imports local files
import { Store } from '../../../../Store';
import { postRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import Unauthorized from '../Unauthorized';
import SnackBar from '../../../../Components/SnackbarComponent';
//router
import { Link } from 'react-router-dom';

const CreateQuestion = (props) => {
    const { state, dispatch } = useContext(Store);

    const [status, setStatus] = useState('');
    const [description, setDescription] = useState('');

    const [isLoading, setIsLoading] = useState(false)

    // Handle error in name
    const [errorInStatus, setErrorInStatus] = useState({
        error: false,
        message: '',
    })

    // Handle error in Country 
    const [errorInDescription, setErrorInDescription] = useState({
        error: false,
        message: '',
    })

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    const DescriptionHandler = (e) => {
        if (errorInDescription.error) {
            setErrorInDescription({
                error: false,
                message: ''
            })
        }
        setDescription(e.target.value)
    }

    // verify if the given text is empty
    const isEmpty = (text) => {
        return text.length === 0 ? true : false;
    }

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    const CheckUserPermission = () => {
        let canUserEdit = false; 

        if (state.userIsLoggedIn) {
            const roles = [...state.currentUser.roles];
            for (let i = 0; i < roles.length; i++)
                if (roles[i].name === 'admin')
                    canUserEdit = true; 
        }
        else {
            canUserEdit = false;
        } 

        return canUserEdit; 
    }


    //Handle submit 
    async function onSubmit() {
        setIsLoading(true)
        if (!isEmpty(status) && !isEmpty(description)) {
            const api = '/questions'
            const body = {
                "question": {
                    'status': status,
                    'description': description,
                }
            }
            postRequest(
                api,
                body,
                (data) => {
                    HandleSnack('A pergunta foi criada com sucesso', true, 'success', '#228B22')
                    setIsLoading(false)
                },
                (error) => {
                    HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
                    setIsLoading(false)
                }
            )
        } else {
            HandleSnack('Você precisa preencher algumas informações obrigatórias', true, 'warning', '#FFC125')
            if (isEmpty(status)) {
                setErrorInStatus({
                    error: true,
                    message: 'Esse campo está vazio'
                })
            }
            if (isEmpty(description)) {
                setErrorInDescription({
                    error: true,
                    message: 'Esse campo está vazio'
                })
            }
            setIsLoading(false)
        }
    }

    //handle change of staus

    const handleChange = (e) => {
        const value = e.target.value;
        setStatus(value);
        console.log(status)
    };

    // Fields
    const fields = [
        {
            label: 'Descrição',
            value: description,
            required: true,
            error: errorInDescription.error,
            errorMessage: errorInDescription.message,
            onChange: (event) => DescriptionHandler(event)
        }
    ]

    const STATUS_OPTIONS = [
        {
            value: "active",
            label: "Ativo",
        },
        {
            value: "inactive",
            label: "Inativo",
        },
    ];

    if(CheckUserPermission()) {
        return (
            <Card>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() => setSnackInfo({
                        message: '',
                        icon: '',
                        open: false,
                        color: ''
                    })}
                />
                <CardContent>
                    <Grid container direction='row' justify='space-between' alignContent="center" alignItems="center" xs={12}>
                        <Grid item>
                            <Typography variant='h4'>
                                Nova question
                            </Typography>
                        </Grid>
                        <Grid item>
                            <Link to={'/admin/Questions'} style={{ textDecoration: 'none' }}>
                                <Button
                                    onClick={props.BackToList}
                                    startIcon={<ListRoundedIcon />}
                                    variant='outlined'
                                    color='primary'
                                >
                                    Listar
                                </Button>
                            </Link>
                        </Grid>
                    </Grid>
    
                    <div style={{ height: '1em' }}></div>
    
                    <form style={{ display: 'flex', flexDirection: 'column' }}>
                        <>
                            <TextField
                                select
                                label="Status"
                                value={status ? status : ""}
                                error={errorInStatus.error}
                                style={{ width: '250px', marginBottom: '1em' }}
                                helperText={errorInStatus.error ? errorInStatus.errorMessage : ''}
                                onChange={handleChange}
                            >
                                {STATUS_OPTIONS.map((option, index) => (
                                    <MenuItem
                                        key={option.value}
                                        value={option.value}
                                        style={option.value === status ? { color: 'blue' } : { color: 'black' }}
                                    >
                                        {
                                            option.label
                                        }
                                    </MenuItem>
                                ))}
                            </TextField>
                            {fields.map((field, index) => (
                                <TextField
                                    key={index}
                                    required={field.required}
                                    error={field.error}
                                    helperText={field.error ? field.errorMessage : ''}
                                    style={{ width: '250px', marginBottom: '1em' }}
                                    label={field.label}
                                    value={field.value}
                                    onChange={field.onChange}
                                    type="search"
                                    multiline={true}
                                />
                            ))}
                        </>
                    </form>
                </CardContent>
                <CardAction>
                    <Button
                        onClick={() => {
                            onSubmit();
                        }}
                        variant="contained"
                        color="primary"
                        disabled={isLoading}
                        startIcon={isLoading ? null : <AddRoundedIcon />}
                    >
                        {
                            isLoading ? <CircularProgress size={24} /> : 'Adicionar'
                        }
                    </Button>
                </CardAction>
            </Card>
        );
    } else return <Unauthorized/>
}

export default CreateQuestion;