/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useContext } from 'react';
//imports from material ui
import clsx from 'clsx';
import { makeStyles } from '@material-ui/core/styles';
import Drawer from '@material-ui/core/Drawer';
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import IconButton from '@material-ui/core/IconButton';
import DisplayIcon from '../../Components/Components/DisplayIcon';
import ListItemText from '@material-ui/core/ListItemText';
import MenuIcon from '@material-ui/icons/Menu';
import Fab from '@material-ui/core/Fab';
//imports from routers
import { BrowserRouter, Switch, Route, Link } from 'react-router-dom';
//imports from local files
import { TabsItens } from '../AdminLabelTabs/LabelTabs';
import Welcome from '../../Components/Components/Welcome';
import NoteVariables from './SubPages/NoteVariables';
import Institution from './SubPages/Institutions';
import SendEmail from './SubPages/SendEmail';
import Inframe from './SubPages/Inframe';
import Languages from './SubPages/Languages';
import Activity from './SubPages/Activity';
import InstitutionCard from '../../Components/Components/DataCards/InstitutionsCard';
import InstitutionsInput from '../../Components/Components/Inputs/IntitutionsInputs';
import CreateInstitution from '../../Components/Components/Inputs/CreateInstitution';
import NoteVarCard from '../../Components/Components/DataCards/NoteVarCard';
import NoteVarInputs from '../../Components/Components/Inputs/NoteVarInputs';
import EditLanguage from '../../Components/Components/Inputs/EditLanguage';
import CreateLanguage from '../../Components/Components/Inputs/CreateLanguage';

import ActivityCard from '../../Components/Components/DataCards/ActivityCard';

import CommunityQuestions from './SubPages/CommunityQuestions';
import CommunityCard from '../../Components/Components/DataCards/CommunityQuestionCard';

import Collections from './SubPages/Collections';
import CollectionCard from '../../Components/Components/DataCards/CollectionCard';
import EditCollection from '../../Components/Components/Inputs/EditCollection';

import Ratings from './SubPages/Rating';
import RatingCard from '../../Components/Components/DataCards/RatingCard';
import EditRating from '../../Components/Components/Inputs/EditRating';
import CreateRating from '../../Components/Components/Inputs/CreateRating';

import Questions from './SubPages/Questions';
import CreateQuestions from '../../Components/Components/Inputs/CreateQuestion';

import EducationalObject from './SubPages/EducationalObjects';
import EducationalObjectCard from '../../Components/Components/DataCards/EducationalObjectsCard';
import EducationalObjectEdit from '../../Components/Components/Inputs/EditEducationalObect';

import Complaints from './SubPages/Complaints';
import ComplaintCard from '../../Components/Components/DataCards/ComplaintsCard';

import AproveTeacher from './SubPages/AproveTeacher';
import UserCard from '../../Components/Components/DataCards/UserCard';
import EditUser from '../../Components/Components/Inputs/EditUser';
import UserList from './SubPages/Users';
import UserPermissions from './SubPages/Permissions'
import EditRole from '../../Components/Components/Inputs/EditRoles';
import CreateRole from '../../Components/Components/Inputs/CreateRole'
import BlockedUser from './SubPages/BlockedUsers';

const useStyles = makeStyles({
    list: {
        width: 250,
    },
    fullList: {
        width: 'auto',
    },
});

const fab = {
    margin: 0,
    top: 'auto',
    right: 20,
    bottom: 20,
    left: 'auto',
    position: 'fixed',
}

export default function Admin() {
    const classes = useStyles();

    //State of the Drawer
    const [State, setState] = React.useState({
        left: false
    });

    {/**************** Controlls the state of the Drawer ****************/ }
    const toggleDrawer = (anchor, open) => (event) => {
        if (event.type === 'keydown' && (event.key === 'Tab' || event.key === 'Shift')) {
            return;
        }

        setState({ ...State, [anchor]: open });
    };

    {/**************** Dsiplay the itens of the Drawer ****************/ }
    const list = (anchor) => (
        <div
            className={clsx(classes.list, {
                [classes.fullList]: anchor === 'top' || anchor === 'bottom',
            })}
            role="presentation"
            onClick={toggleDrawer(anchor, false)}
            onKeyDown={toggleDrawer(anchor, false)}
        >
            <List>
                {TabsItens.map((text, index) => (
                    <Link to={text.href} key={text.label} style={{ color: "black" }}>
                        <ListItem button key={text.label}>
                            <IconButton>
                                <DisplayIcon i={index} />
                            </IconButton>
                            <ListItemText primary={text.label} />
                        </ListItem>
                    </Link>
                ))}
            </List>
        </div>
    );

    return (
        <BrowserRouter>
            <Switch>
                <div style={{ paddingTop: '2em', paddingLeft: '2em', paddingRight: '2em', paddingBottom: '2em', backgroundColor: '	#D3D3D3' }}>
                    <Route path='/admin/home' exact={true} component={Welcome}/>

                    <Route path='/admin/intitutions' component={Institution} />
                    <Route path='/admin/institution/:id' component={InstitutionCard} />
                    <Route path='/admin/institutionEdit/:id' component={InstitutionsInput}/>
                    <Route path='/admin/InstitutionCreate' component={CreateInstitution} />

                    <Route path='/admin/noteVars' component={NoteVariables} />
                    <Route path='/admin/noteVar/:id' component={NoteVarCard} />
                    <Route path='/admin/noteVarEdit/:id' component={NoteVarInputs} />

                    <Route path='/admin/languages' component={Languages} />
                    <Route path='/admin/languageEdit/:id' component={EditLanguage} />
                    <Route path='/admin/languageCreate' component={CreateLanguage} />

                    <Route path='/admin/CommunityQuestions' component={CommunityQuestions} />
                    <Route path='/admin/CommunityQuestion/:id' component={CommunityCard} />

                    <Route path='/admin/Collections' component={Collections} />
                    <Route path='/admin/Collection/:id' component={CollectionCard} />
                    <Route path='/admin/EditCollection/:id' component={EditCollection} />

                    <Route path='/admin/Ratings' component={Ratings} />
                    <Route path='/admin/Rating/:id' component={RatingCard} />
                    <Route path='/admin/EditRating/:id' component={EditRating} />
                    <Route path='/admin/CreateRating' component={CreateRating} />

                    <Route path='/admin/Questions' component={Questions} />
                    <Route path='/admin/CreateQuestion' component={CreateQuestions} />

                    <Route path='/admin/activities' component={Activity} />
                    <Route path='/admin/activity/:id' component={ActivityCard} />

                    <Route path='/admin/learningObjects' component={EducationalObject} />
                    <Route path='/admin/learningObject/:id' component={EducationalObjectCard} />
                    <Route path='/admin/learningObjectEdit/:id' component={EducationalObjectEdit} />

                    <Route path='/admin/complaints' component={Complaints} />
                    <Route path='/admin/complaint/:id' component={ComplaintCard} />

                    <Route path='/admin/users/teacher_requests' component={AproveTeacher} />
                    <Route path='/admin/usersList' component={UserList} />
                    <Route path='/admin/user/:id' component={UserCard} />
                    <Route path='/admin/EditUser/:id' component={EditUser} />
                    <Route path='/admin/permissions' component={UserPermissions} />
                    <Route path='/admin/EditPermissions/:id' component={EditRole} />
                    <Route path='/admin/CreateRole' component={CreateRole} />
                    <Route path='/admin/BlockedUsers' component={BlockedUser} />

                    <Route path='/admin/sendEmail/:email' component={SendEmail} />
                    <Route path='/admin/inframe' component={Inframe} />

                </div>
            </Switch>

            <React.Fragment>
                <Drawer anchor={'left'} open={State['left']} onClose={toggleDrawer('left', false)}>
                    {list('left')}
                </Drawer>
            </React.Fragment>
            <Fab color="primary" aria-label="add" style={fab} onClick={toggleDrawer('left', true)}>
                <MenuIcon />
            </Fab>
        </BrowserRouter>
    );
}
